-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
deferred class DISPLAY

inherit
	IMAGE -- A display is an image, so it can be blitted and blitted to.

feature {NONE} -- Initialization

	make_display is
		do
			create redraw_list.make (1, 0)
			create root.make_root (Current)
			create event_queue.make
		end

feature -- Deferred operations

	poll_event is
			-- Poll for an event and put it in the queue
		deferred
		end

	refresh (region: RECTANGLE) is
			-- Update `region' after graphical operations
		deferred
		end

	capture_image (region: RECTANGLE) is
			-- Get image from `region', store in `created_image'
		deferred
		end

	close is
			-- Close display
		deferred
		end

	created_image: IMAGE
		-- Last created image

feature -- Access

	root: WINDOW
		-- Display's root window

	dimensions: RECTANGLE
		-- Display dimensions

	event: EVENT is
			-- First unprocessed event
		require
			event_available
		do
			Result := event_queue.first
		end

	event_available: BOOLEAN is
			-- Are there unprocessed events?
		do
			Result := not event_queue.is_empty
		end

	default_font: FONT
		-- Font used for widgets when its font is not set

feature -- Operations

	set_default_font (new_font: FONT) is
			-- Change `default_font'
		do
			default_font := new_font
		ensure
			default_font = new_font
		end

feature -- Operations -- Events

	wait_event is
			-- Wait for an event and put it in the queue
			-- Redefine if possible to avoid poll&wait
		do
			from until event_available loop poll_event end
		ensure
			event_available
		end

	next_event is
			-- Removes first event from the queue
		require
			event_available
		do
			event_queue.remove_first
		end

	add_event (new_event: EVENT) is
			-- add `new_event' to the event queue
		require
			new_event /= Void
		do
			event_queue.add_last (new_event)
		end

	do_event_loop is
			-- Enter event handling loop
		local
			evloopsave: BOOLEAN
		do
			-- Save state
			evloopsave := end_event_loop

			-- Event loop
			end_event_loop := False
			from until end_event_loop loop
				do_event_loop_iteration (True)
			end

			-- Restore state
--Check that this works ok with nested event loops
			event_queue.make -- Instead of clear, to avoid problems with SE2
			end_event_loop := evloopsave
		ensure
			end_event_loop = old end_event_loop
		end

		do_event_loop_iteration (wait: BOOLEAN) is
			-- A single iteration of the event handling loop
			-- Waits for an event if `wait'
		do
			if wait then
				wait_event
			else
				poll_event
			end
			from until not event_available loop
				event_capture
				if not event.handled then
					root.handle_event (event)
				end
				if not event.handled then
					default_event_handler
				end
				poll_event
				next_event
			end
			redraw
		end

	event_capture is
			-- Called for all events, before dispatching
		require
			event_available
		local
			m: EVENT_MOUSE_MOVE
		do
			m ?= event
			if m /= Void then
				pointer_x := m.x
				pointer_y := m.y
			end
		ensure
			event_available
		end

	default_event_handler is
			-- Called for all unprocessed events
		require
			event_available
		local
			q: EVENT_QUIT
		do
			q ?= event
			if q /= Void then
				event.set_handled
				end_event_loop := True
			end
		end

feature -- Access -- Pointer

	pointer_x, pointer_y: INTEGER
		-- Pointer coordinate

feature -- Operations -- Pointer

	update_pointer is
			-- Show pointer again
		local
			p: MOUSE_POINTER
		do
			p := root.subwindow_at (pointer_x, pointer_y).pointer
			if p /= Void then
				set_pointer (p, pointer_x, pointer_y)
			end
		end

feature {NONE} -- Internal -- Pointer

	set_pointer (pointer: MOUSE_POINTER; x,y: INTEGER) is
			-- (re)Display `pointer' at (`x',`y')
		require
			pointer /= Void
		local
			oldr, newr: RECTANGLE
		do
			if pointer_save /= Void then
				oldr := saved_area
				hide_pointer
			else
				oldr.set_with_size (x, y, 0, 0)
			end
			pointer_x := x
			pointer_y := y
			show_pointer (pointer.image, x-pointer.hotspot_x, y-pointer.hotspot_y)
			newr.set_with_size (x-pointer.hotspot_x, y-pointer.hotspot_y,
				pointer.image.width, pointer.image.height)
			refresh ((oldr+newr)*dimensions)
		end

	hide_pointer is
			-- Hide pointer, recovering image saved
		require
			pointer_save /= Void
		do
			pointer_save.show (Current, saved_area.x, saved_area.y)
			pointer_save := Void
		ensure
			pointer_save = Void
		end

	show_pointer (pointer: IMAGE; x,y: INTEGER) is
			-- Show the `pointer' at (`x',`y') saving the screen behind
		require
			pointer /= Void
			pointer_save = Void
		local
			ptr_region: RECTANGLE
		do
			saved_area.set_with_size (x, y, pointer.width, pointer.height)
			saved_area := saved_area * dimensions
			-- Save background
			capture_image (saved_area)
			pointer_save := created_image
			created_image := Void
			-- Draw pointer
			ptr_region := saved_area
			ptr_region.translate (-x, -y)
			pointer.show_region (Current, ptr_region, saved_area.x, saved_area.y)
		ensure
			pointer_save /= Void
		end

	pointer_save: IMAGE
		-- Copy of display under pointer

	saved_area: RECTANGLE
		-- Screen area saved in `pointer_save'

feature {WINDOW} -- Operations -- redrawing

	request_redraw (area: RECTANGLE) is
			-- Request a redraw for `area'
		local
			a, s, o: RECTANGLE
			i: INTEGER
		do
			from
				a := area
				i := redraw_list.lower
			until i > redraw_list.upper loop
				o := redraw_list @ i
				s := a + o
				if s.area/(a.area+o.area) < alpha then
					a := s
					redraw_list.put (redraw_list.last, i)
					redraw_list.remove_last
					i := redraw_list.lower
				else
					i := i + 1
				end
			end
			redraw_list.add_last (a)
		end

	redraw is
			-- Called for every event batch. Redraw requested screen area.
		local
			r: RECTANGLE
			i: INTEGER
		do
			if pointer_save /= Void then
				r := saved_area
				hide_pointer
			end
			if redraw_list.count > 0 and root.visible then
				from i := redraw_list.lower until i > redraw_list.upper loop
					root.redraw (redraw_list @ i)
					i := i + 1
				end
			end
			update_pointer
			from i := redraw_list.lower until i > redraw_list.upper loop
				refresh (redraw_list @ i)
				i := i + 1
			end
			redraw_list.make (1,0) -- Instead of clear, to avoid problems with SE2
			refresh (r)
		end

feature {NONE} -- Operations -- redrawing (internal)

	redraw_list: ARRAY [RECTANGLE]
		-- areas that must be redrawn

	alpha: REAL is 3
		-- Factor of loss required to not merge to redraw areas

feature {NONE} -- Representation

	end_event_loop: BOOLEAN
		-- Flag to mark end of event loop

	event_queue: LINKED_LIST [EVENT]
		-- FIFO queue of incoming events (in chronological order)

end -- class DISPLAY