/*
-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
--  This file contains some code derived from JEGL, distributable under
--  the same license, available at http://jegl.sourceforge.net,
--  Copyright (C) 2000 Jan Nockemann
*/
#include <stdlib.h>
#include <string.h>
#include <SDL.h>

#define BOOLEAN char
#define abs(x) ((x) > 0 ? (x) : -(x))
#define toggle(x) (x ? 0 : 1)


static Uint32 current_interval = 0 ;
static Uint32 timer_count = 0 ;

static Uint32 SDL_DISPLAY_timer_callback (Uint32 interval) {

    char canpush = 1 ;
    SDL_Event ev ;

    timer_count++ ;
    while (timer_count > 0 && canpush == 1) {
        ev.type = SDL_USEREVENT ;
        ev.user.code = timer_count ;
        canpush = SDL_PushEvent ((SDL_Event *)&ev) ;
        if (canpush == 1)
            timer_count-- ;
    }

    return current_interval ;
}

void SDL_DISPLAY_set_interval (Uint32 new_interval) {
    current_interval = new_interval ;
    SDL_SetTimer (current_interval, SDL_DISPLAY_timer_callback) ;
}

void SDL_SURFACE_put_pixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

void SDL_SURFACE_fillrect (SDL_Surface *ptr, int x, int y, int w, int h,
                           Uint32 color) {
    SDL_Rect r ;
    r.x = x; r.y = y;
    r.w = w; r.h = h;
    SDL_FillRect (ptr, &r, color) ;
}

static void _put_this_pixel_8bpp(Uint8 *pixels, Uint32 color)
{
    *pixels = color;
}

static void _put_this_pixel_16bpp(Uint8 *pixels, Uint32 color)
{
    *(Uint16 *)pixels = color;
}

static void _put_this_pixel_24bpp(Uint8 *pixels, Uint32 color)
{
    if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
    {
        pixels[0] = (color >> 16) & 0xff;
        pixels[1] = (color >> 8) & 0xff;
        pixels[2] = color & 0xff;
    }
    else
    {
        pixels[0] = color & 0xff;
        pixels[1] = (color >> 8) & 0xff;
        pixels[2] = (color >> 16) & 0xff;
    }
}

static void _put_this_pixel_32bpp(Uint8 *pixels, Uint32 color)
{
    *(Uint32 *)pixels = color;
}

/* Implements the Bresenham Line-Drawing Algorithm */
void SDL_SURFACE_drawline (SDL_Surface *surface, int x1, int y1, int x2, int y2,
                           int size, const int *dash, int offset, Uint32 color)
{

    int i, j = 0, period = 0, is_on = 1;
    int dx = x2 - x1, dy  = y2 - y1;
    int eps = 0, mineps, lildeps, bigdeps;
    int bpp = surface->format->BytesPerPixel;
    int xx, xx1, xx2, dxx;
    int deltax, deltay;
    Uint8 *ptr = (Uint8 *)surface->pixels + y1 * surface->pitch + x1 * bpp;
    void (*put_this_pixel)(Uint8*, Uint32) = NULL;
    /*    printf("Called to draw line (%d, %d)(%d, %d)\n", x1, y1, x2, y2);*/
    /*    printf("Dash = [");
    for(i = 0; i < size; i++)
      printf("%d, ", dash[i]);
    printf("]\n");
    */

    /* Choose the right pixel-putting function */
    switch(bpp) {
    case 1:
        put_this_pixel = _put_this_pixel_8bpp;
        break;
    case 2:
        put_this_pixel = _put_this_pixel_16bpp;
        break;
    case 3:
        put_this_pixel = _put_this_pixel_24bpp;
        break;
    case 4:
        put_this_pixel = _put_this_pixel_32bpp;
        break;
    default:
        fprintf (stderr, "Invalid bpp value\n");
        abort();
    }


    /* Jump to starting offset */
    if (size)
    {
        for(i = 0; i < size; i++)
            period += dash[i];
        offset = offset % period;
        j = 0;
        while (dash[j] < offset)
        {
            offset -= dash[j];
            j++;
            is_on = toggle(is_on);
            if (j==size)
                j = 0;
        }
        i = offset;
    }

    /* Configure algorithm according to actual octant */
    if (abs(y2 - y1) > abs(x2 - x1))
    {
        xx = y1;
        if (x2 > x1) { lildeps = dx; deltay = bpp; }
        else { lildeps = -dx; deltay = -bpp; }
        if (y2 > y1) { bigdeps = dy; mineps = dy; xx1 = y1; xx2 = y2; dxx = 1; deltax = surface->pitch; }
        else { bigdeps = -dy; mineps = -dy; xx1 = y2; xx2 = y1; dxx = -1; deltax = -(surface->pitch); }
    }
    else
    {
        xx = x1;
        if (y2 > y1) { lildeps = dy; deltay = surface->pitch; }
        else { lildeps = -dy; deltay = -(surface->pitch); }
        if (x2 > x1) { bigdeps = dx; mineps = dx; xx1 = x1; xx2 = x2; dxx = 1; deltax = bpp; }
        else { bigdeps = -dx; mineps = -dx; xx1 = x2; xx2 = x1; dxx = -1; deltax = -bpp; }
    }

    if (SDL_MUSTLOCK(surface))
        SDL_LockSurface(surface);

    /* The real thing */
    /*printf("for x := %d to (%d, %d) step %d\n", xx, xx1, xx2, dxx);*/
    for (; (xx1 <= xx) && (xx <= xx2); xx += dxx, ptr += deltax)
    {
        /*printf("ptr=%p, x = %d, y = %d\n", ptr, xx, (((Uint32)ptr) - ((Uint32)(surface->pixels))) / surface->pitch);*/
        if (is_on)
            put_this_pixel(ptr, color);
        eps += lildeps;
        /*printf("eps = %d, flat in (%d, %d).\n", (eps<<1), -mineps, mineps);*/
        if ((-mineps >= (eps << 1)) || ((eps << 1) >= mineps))
        {
            ptr += deltay;
            eps -= bigdeps;
            /*printf("Went diagonal.  Now eps=%d\n", eps);*/
        }
    /* That's it, now just check is_on */
        if (size)
        {
            if (dash[j] == i)
            {
                i = 0;
                if (j == size - 1)
                {
                    j = 0;
                    is_on = 1;
                }
                else
                {
                    j++;
                    is_on = toggle(is_on);
                }
            }
            else
                i++;
        }
    }

    if (SDL_MUSTLOCK(surface))
        SDL_UnlockSurface(surface);
}



Uint32 SDL_SURFACE_get_pixel (SDL_Surface * surface, int x, int y)
{
    void *bits;
    Uint32 bpp;

    bpp = surface->format->BytesPerPixel;
    bits = ((Uint8 *) surface->pixels) + y * surface->pitch + x * bpp;

    switch (bpp){
        case 2: return *((Uint16 *) bits);
        case 3: {
            Uint32 r;
            r = *((Uint32 *)bits) ;
            /* This only works on little endian architectures */
            r = r & 0x00ffffff ;
            return r;
        }
        case 4: return *((Uint32 *)bits);
    }
    return (-1);
}

int SDL_SURFACE_colorize (SDL_Surface *img, int x, int y, int w, int h, int size, Uint32 *palette)
{
    int i, j, maxx, maxy, offset, Bpp, bpp;
    Uint16 pixel;

    Bpp = img->format->BytesPerPixel;
    bpp = img->format->BitsPerPixel;
    if ((img == NULL) || (palette == NULL))
        return 1;
    maxx = (x + w < img->w ? x + w : img->w);
    maxy = (y + h < img->h ? y + h : img->h);
    SDL_LockSurface(img);

    for (i = y; i < maxy; i++)
    {
        offset = i * img->pitch + Bpp * x;
        for (j = x; j < maxx; j++)
        {
            pixel = (*((Uint32*)(((int)img->pixels) + offset)))>>(32 - bpp);
            if ((0 < pixel) && (pixel < size))
            {
                ((Uint8*)(img->pixels))[offset + 0] = (Uint8) (palette[pixel]);
                ((Uint8*)(img->pixels))[offset + 1] = (Uint8) (palette[pixel] >> 8);
            }
            offset += Bpp;
        }
    }
    SDL_UnlockSurface(img);
    return 0;
}

/* Taken from JEGL */

SDL_Surface *SDL_SURFACE_create (SDL_Surface *screen, int w, int h, BOOLEAN transp)
{
  SDL_Surface *surface, *converted;

  surface = SDL_CreateRGBSurface (SDL_HWSURFACE, w, h,
                                  screen->format->BitsPerPixel, 0, 0, 0, 0);
  if (surface == NULL)
    {
      fprintf (stderr, "Couldn't create surface: %s\n", SDL_GetError ());
      return (NULL);
    }

  SDL_LockSurface (surface) ;
  if (transp)
      converted = SDL_DisplayFormatAlpha (surface);
  else
      converted = SDL_DisplayFormat (surface);
  SDL_UnlockSurface (surface) ;
  SDL_FreeSurface (surface);
  if (converted == NULL)
    {
      fprintf (stderr, "Couldn't convert surface: %s\n", SDL_GetError ());
      return (NULL);
    }

  memset(converted->pixels, 0, converted->pitch * h);
  return (converted);
}

void SDL_SURFACE_blit (SDL_Surface *src, SDL_Surface *dst,
                       int srcX, int srcY, int dstX, int dstY,
                       int wid, int hgt)
{
  SDL_Rect src_rect, dst_rect;

  src_rect.x = srcX;
  src_rect.y = srcY;
  src_rect.w = wid;
  src_rect.h = hgt;

  dst_rect.x = dstX;
  dst_rect.y = dstY;
  dst_rect.w = wid;
  dst_rect.h = hgt;

  SDL_BlitSurface (src, &src_rect, dst, &dst_rect);
}

SDL_Surface *SDL_SCREEN_set_videomode (int w, int h, int bpp, int fullscreen)
{
  SDL_Surface *screen;

  const SDL_VideoInfo *info;
  Uint32 flags = SDL_SWSURFACE;

  if (SDL_Init (SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE) < 0)
    {
      fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
      exit (1);
    }

  if (fullscreen)
    {
      flags |= SDL_FULLSCREEN;

      info = SDL_GetVideoInfo ();
      if (info->blit_hw_CC && info->blit_fill)
	{
	  flags |= SDL_HWSURFACE;
	  flags |= SDL_DOUBLEBUF;
	}
    }

  screen = SDL_SetVideoMode (w, h, bpp, flags);
  if (screen == NULL)
    {
      fprintf (stderr, "Couldn't set display mode: %s\n", SDL_GetError ());
      exit (1);
    }

  SDL_EnableUNICODE (1);
  return screen;
}
