-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class SDL_DISPLAY

inherit
	DISPLAY
		redefine wait_event, hide_pointer end
	SDL_CONSTANTS
	SDL_IMAGE
		rename
			make as img_make
		end

create
	make

feature -- Creation

	make (w, h: INTEGER; bpp: INTEGER; full_screen: BOOLEAN) is
			-- New display of `w'x`h' pixels at `bpp' bits per pixel
			-- May be `full_screen'
		require
			not_videomode_set: not is_videomode_set
			valid_size: w > 0 and h > 0
			valid_bpp: bpp = 16 or bpp = 24 or bpp = 32
		local
			showcursor: INTEGER
		do
			set_videomode (w, h, bpp, full_screen)
			showcursor := SDL_ShowCursor (0)
			dimensions.set_with_size (0, 0, w, h)
			to_external := screen.to_external
			dont_release_flag := True
			make_display
		end

feature -- Operations

	close is
			-- Close display
		local
			sdl: SDL
		do
			sdl.quit
		end

	poll_event is
		local
			s_event: SDL_EVENT
		do
			create s_event.make
			if s_event.poll then
				process_sdl_event (s_event)
			end
		end

	wait_event is
		local
			s_event: SDL_EVENT
		do
			create s_event.wait
			process_sdl_event (s_event)
		end

	capture_image (region: RECTANGLE) is
			-- Get image from `region', store in `created_image'.
		do
			create {SDL_IMAGE}created_image.make_from_screen (region)
		end

	set_timer_interval (delay: INTEGER) is
			-- generate timer events each `delay' ms.
		require
			delay >= 0
		external "C"
		alias "SDL_DISPLAY_set_interval"
		end

feature {NONE} -- Internal

	process_sdl_event (s_event: SDL_EVENT) is
			-- Convert `s_event' to EWS event and put into event queue.
		local
			new_event: EVENT
		do
			inspect s_event.type
			when event_quit then
				create {EVENT_QUIT}new_event
			when event_mousemotion then
				create {EVENT_MOUSE_MOVE}new_event.make (s_event.mouse_x, s_event.mouse_y)
			when event_mousebuttondown, event_mousebuttonup then
				create {EVENT_MOUSE_BUTTON}new_event.make (s_event.button_x,
					s_event.button_y, s_event.button,
					s_event.type = event_mousebuttondown)
			when event_keyup, event_keydown then
				create {EVENT_KEY}new_event.make (s_event.unicode,
					s_event.sym, s_event.mod,
					s_event.type = event_keydown)
			when event_timerevent then
				create {EVENT_TIMER}new_event
			else
				create {EVENT_SDL_CUSTOM}new_event.make (s_event)
			end
			check new_event /= Void end
			event_queue.add_last (new_event)
		end

	hide_pointer is
			-- This is redefined to save some memory now instead of wainting to the GC
		local
			p: SDL_IMAGE
		do
			p ?= pointer_save
			Precursor
				check p /= Void end
			p.dispose
		end

feature {NONE} -- SDL external functions

	SDL_ShowCursor (status: INTEGER): INTEGER is
			-- Toggle cursor, return old cursor state; check SDL docs
		require
			status.in_range (-1, 1)
		external "C"
		alias "SDL_ShowCursor"
		ensure
			Result.in_range (0, 1)
		end

end -- class SDL_DISPLAY