-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class SDL_EVENT

inherit
	MEMORY
		redefine dispose end
	SDL_CONSTANTS

create
	make, wait

feature {NONE} -- Creation

	make is
			-- New empty SDL_event
		do
			to_external := ext_alloc_event
		end

	wait is
			-- New event from queue (blocks if needed)
		local
			dummy: INTEGER
		do
			make
			dummy := SDL_WaitEvent (to_external)
			check dummy = 1 end
		end

feature -- Access

	type: INTEGER is
			-- Event kind
		do
			Result := ext_event_type (to_external)
		end

feature -- Access -- SDL_MouseMotionEvent

	mouse_x: INTEGER is
			-- Horizontal mouse position
		require
			type = event_mousemotion
		do
			Result := ext_event_mousemotion_x (to_external)
		end

	mouse_y: INTEGER is
			-- Vertical mouse position
		require
			type = event_mousemotion
		do
			Result := ext_event_mousemotion_y (to_external)
		end

feature -- Access -- SDL_MouseButtonEvent

	button: INTEGER is
			-- Button clicked/released
		require
			type = event_mousebuttonup or type = event_mousebuttondown
		do
			Result := ext_event_mousebutton (to_external)
		end

	button_x: INTEGER is
			-- Horizontal mouse position on click/release
		require
			type = event_mousebuttonup or type = event_mousebuttondown
		do
			Result := ext_event_mousebutton_x (to_external)
		end

	button_y: INTEGER is
			-- Vertical mouse position on click/release
		require
			type = event_mousebuttonup or type = event_mousebuttondown
		do
			Result := ext_event_mousebutton_y (to_external)
		end

feature -- Access -- SDL_KeyboardEvent

	sym: INTEGER is
			-- Key pressed/released
		require
			type = event_keydown or type = event_keyup
		do
			Result := ext_event_keyboard_sym (to_external)
		end

	mod: INTEGER is
			-- Modifiers on last keypress/release
		require
			type = event_keydown or type = event_keyup
		do
			Result := ext_event_keyboard_mod (to_external)
		end

	unicode: INTEGER is
			-- Unicode of last keypress/release
		require
			type = event_keydown or type = event_keyup
		do
			Result := ext_event_keyboard_unicode (to_external)
		end

feature -- Operations

	poll: BOOLEAN is
			-- Event available in queue. Update Current when true
			-- WARNING: has side effects
		local
			polled: INTEGER
		do
			make
			polled := SDL_PollEvent (to_external)
			Result := polled = 1
		end

feature -- Redefined features

	dispose is
			-- Release external memory
		do
			ext_release_event (to_external)
		end

feature {NONE} -- Representation

	to_external: POINTER

feature {NONE} -- External functions -- Event allocation

	ext_alloc_event: POINTER is
		external "C inline use <stdlib.h>"
		alias "(malloc (sizeof (SDL_Event)))"
		ensure
			not Result.is_null
		end

	ext_release_event (ext: POINTER) is
		require
			not ext.is_null
		external "C inline use <stdlib.h>"
		alias "free ( $ext )"
		end

feature {NONE} -- External functions -- Event access

	ext_event_type (ext: POINTER): INTEGER is
		require
			not ext.is_null
		external "C inline use <SDL.h>"
		alias "((int)(((SDL_Event *) $ext) -> type))"
		end
	
	ext_event_mousemotion_x (ext: POINTER): INTEGER is
		require
			not ext.is_null
			ext_event_type (ext) = event_mousemotion
		external "C inline use <SDL.h>"
		alias "((int)(((SDL_Event *) $ext) -> motion.x))"
		end

	ext_event_mousemotion_y (ext: POINTER): INTEGER is
		require
			not ext.is_null
			ext_event_type (ext) = event_mousemotion
		external "C inline use <SDL.h>"
		alias "((int)(( (SDL_Event *) $ext) -> motion.y))"
		end

	ext_event_mousebutton (ext: POINTER): INTEGER is
		require
			not ext.is_null
				ext_event_type (ext) = event_mousebuttonup
			or
				ext_event_type (ext) = event_mousebuttondown
		external "C inline use <SDL.h>"
		alias "((int)(( (SDL_Event *) $ext) -> button.button))"
		end

	ext_event_mousebutton_x (ext: POINTER): INTEGER is
		require
			not ext.is_null
				ext_event_type (ext) = event_mousebuttonup
			or
				ext_event_type (ext) = event_mousebuttondown
		external "C inline use <SDL.h>"
		alias "((int)(( (SDL_Event *) $ext) -> button.x))"
		end

	ext_event_mousebutton_y (ext: POINTER): INTEGER is
		require
			not ext.is_null
				ext_event_type (ext) = event_mousebuttonup
			or
				ext_event_type (ext) = event_mousebuttondown
		external "C inline use <SDL.h>"
		alias "((int)(( (SDL_Event *) $ext) -> button.y))"
		end

	ext_event_keyboard_sym (ext: POINTER): INTEGER is
		require
			not ext.is_null
				ext_event_type (ext) = event_keyup
			or
				ext_event_type (ext) = event_keydown
		external "C inline use <SDL.h>"
		alias "((int)(((SDL_Event *) $ext)-> key.keysym.sym))"
		end

	ext_event_keyboard_mod (ext: POINTER): INTEGER is
		require
			not ext.is_null
				ext_event_type (ext) = event_keyup
			or
				ext_event_type (ext) = event_keydown
		external "C inline use <SDL.h>"
		alias "((int)(((SDL_Event *) $ext)-> key.keysym.mod))"
		end

	ext_event_keyboard_unicode (ext: POINTER): INTEGER is
		require
			not ext.is_null
				ext_event_type (ext) = event_keyup
			or
				ext_event_type (ext) = event_keydown
		external "C inline use <SDL.h>"
		alias "((int)(((SDL_Event *) $ext) -> key.keysym.unicode))"
		end

feature {NONE} -- External functions -- Event operations

	SDL_WaitEvent (s_event: POINTER): INTEGER is
		require
			not s_event.is_null
		external "C use <SDL.h>"
		alias "SDL_WaitEvent"
		ensure
			Result = 1
		end

	SDL_PollEvent (s_event: POINTER): INTEGER is
		require
			not s_event.is_null
		external "C use <SDL.h>"
		alias "SDL_PollEvent"
		end

invariant
	not to_external.is_null

end -- class SDL_EVENT
