-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class SDL_LINE_IMAGE

inherit
	IMAGE
	SDL_SCREEN_ACCESS

create
	make

feature {NONE} -- Creation

	make (xstart, ystart, xfinish, yfinish, r, g, b: INTEGER) is
			-- New line going from (`xstart', `ystart') to  (`xfinish', `yfinish'),
			-- with color (`r',`g',`b')
		require
			xstart >= 0 and xfinish >= 0 and ystart >= 0 and yfinish >= 0
		do
			x1 := xstart
			y1 := ystart
			x2 := xfinish
			y2 := yfinish
			width := x2.max(x1) + 1
			height := y2.max(y1) + 1
			color := screen.get_color (r, g, b)
			create dash.make(1, 0)
		end

feature -- Access

	is_compatible (img: IMAGE): BOOLEAN is
		local
			image: SDL_IMAGE
		do
			image ?= img
			Result := image /= Void
		end

	width: INTEGER

	height: INTEGER

	offset: INTEGER

feature -- Operations

	show (img: IMAGE; x, y: INTEGER) is
		local
			image: SDL_IMAGE
			r: RECTANGLE
		do
			image ?= img
				r.set_with_size(0, 0, img.width, img.height)
				sdl_show_region(image, r, x, y)
		end

	show_region (img: IMAGE; region: RECTANGLE; x, y: INTEGER) is
		local
			image: SDL_IMAGE
		do
			image ?= img
			sdl_show_region(image, region, x, y)
		end

	set_offset (o: INTEGER) is
		require
			valid_offset: o >= 0
		do
			offset := o
		ensure
			offset = o
		end

	set_color (r, g, b: INTEGER) is
		require
			valid_color: r.in_range(0, 255) and g.in_range(0, 255) and b.in_range(0, 255)
		do
			color := screen.get_color (r, g, b)
		end

	set_dash(d: ARRAY[INTEGER]) is
		require
			valid_dash: d /= Void
		do
			dash := d
		ensure
			dash = d
		end

feature {NONE} -- Representation

	x1, y1, x2, y2: INTEGER

	color: INTEGER

	dash: ARRAY[INTEGER]

feature {NONE} -- Internal

	sdl_show_region (s: SDL_SURFACE; region: RECTANGLE; x, y: INTEGER) is
		local
			a: DOUBLE
			b: DOUBLE
			leftx, rightx, xtra, lefty, righty: INTEGER
			xx1, yy1, xx2, yy2: INTEGER
		do
			xx1 := x + x1 - region.x
			xx2 := x + x2 - region.x
			yy1 := y + y1 - region.y
			yy2 := y + y2 - region.y
			region.set_with_size(x.max(0).min(s.width - 1), y.max(0).min(s.height - 1), region.width.min(s.width - x).max(0), region.height.min(s.height - y).max(0))
				check
						region.x >= 0
						region.y >= 0
						region.x2 <= s.width - 1
						region.y2 <= s.height - 1
				end
			if region.width > 0 and region.height > 0 then
				if x2 = x1 then
					if xx1.in_range(region.x, region.x + region.width) and region.y < yy1.max(yy2) and region.y2 > yy1.min(yy2) then
						s.drawline (xx1, yy1.max(region.y).min(region.y2), xx2, yy2.max(region.y).min(region.y2), dash, offset, color)
					end
				elseif y2 = y1 then
					if yy1.in_range(region.y, region.y + region.height) and region.x < xx1.max(xx2) and region.y2 > xx1.min(xx2) then
						s.drawline (xx1.max(region.x).min(region.x2), yy1, xx2.max(region.x).min(region.x2), yy2, dash, offset, color)
					end
				else
					a := (y2 - y1) / (x2 - x1)
					b := yy1 - a * xx1
					leftx := xx1.min(xx2).max(region.x)  -- calculate intersection of the two rectangles (step 1)
					rightx := xx1.max(xx2).min(region.x2)
					if a > 0 then
						leftx := leftx.max (((region.y - b) / a).rounded) -- (step 2)
						rightx := rightx.min (((region.y2 - b) / a).rounded)
					else
						leftx := leftx.max (((region.y2 - b) / a).rounded)
						rightx := rightx.min (((region.y - b) / a).rounded)
					end
					if rightx > leftx then
						if x2 < x1 then -- swap x1 and x2
							xtra := leftx
							leftx := rightx
							rightx := xtra
						end
						lefty := (a * leftx + b).rounded.min(region.y2).max(region.y)
						righty := (a * rightx + b).rounded.min(region.y2).max(region.y)

						s.drawline (leftx, lefty, rightx, righty, dash, offset, color)
							check
								leftx >= region.x
								rightx >= region.x
								leftx <= region.x2
								rightx <= region.x2
								lefty >= region.y
								righty >= region.y
								lefty <= region.y2
								righty <= region.y2
							end
					end
				end
			end
		end


end -- class SDL_LINE_IMAGE
