-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class SDL_SURFACE

inherit
	SDL_SCREEN_ACCESS
	MEMORY
		redefine dispose end

create
	make, make_transparent, make_from_img, make_from_surface

feature {NONE} -- Creation

	make (new_width, new_height: INTEGER) is
		require
			valid_size: new_width >= 0 and new_height >= 0
			is_videomode_set
		do
			to_external := SDL_SURFACE_create (screen.to_external,
				new_width, new_height, False)
		ensure
			width = new_width
			height = new_height
		end

	make_transparent (new_width, new_height: INTEGER) is
		require
			valid_size: new_width >= 0 and new_height >= 0
			is_videomode_set
		do
			to_external := SDL_SURFACE_create (screen.to_external,
				new_width, new_height, True)
		ensure
			width = new_width
			height = new_height
		end

	make_from_img (filename: STRING) is
		require
			filename_not_void: filename /= Void
			file_exists: file_tools.is_readable (filename)
		do
			to_external := SDL_IMG_Load (filename.to_external)
		end -- make_from_img

	make_from_surface (s: POINTER) is
		require
			not s.is_null
		do
			to_external := s
		end

feature -- Redefined features

	dispose is
		do
			if not dont_release_flag then
				SDL_FreeSurface (to_external)
				dont_release_flag := True
			end
		end

feature -- Access

	width: INTEGER is
			-- Surface width
		do
			Result := SDL_SURFACE_width (to_external)
		end

	height: INTEGER is
			-- Surface height
		do
			Result := SDL_SURFACE_height (to_external)
		end

	bounds: RECTANGLE is
		do
			Result.set_with_size (0, 0, width, height)
		end

	get_pixel (x, y: INTEGER): INTEGER is
		require
-- locking features are not implemented yet
--        locked_if_necessary: must_lock implies is_locked
			location_in_bounds: bounds.has (x, y)
		do
			Result := SDL_SURFACE_get_pixel (to_external, x, y)
		end

	get_color (r, g, b: INTEGER): INTEGER is
			-- value for a pixel with color (`r',`g',`b') in this surface
		do
			Result := SDL_MapRGB (to_external, r, g, b)
		end

feature -- Operations

	blit (dst: SDL_SURFACE; x, y: INTEGER; area: RECTANGLE) is
			-- blit `area' into `dst' at (`x',`y')
		require
			dst_not_void: dst /= Void
		do
			SDL_SURFACE_blit (to_external, dst.to_external, area.x, area.y,
				x, y, area.width, area.height)
		end

	blit_fast (dst: SDL_SURFACE; x, y: INTEGER) is
			-- blit into `dst' at (`x',`y')
		require
			dst_not_void: dst /= Void
		do
			SDL_SURFACE_blit (to_external, dst.to_external, 0, 0,
				x, y, width, height)
		end

	fillrect (r: RECTANGLE; color: INTEGER) is
		do
			SDL_SURFACE_fillrect (to_external, r.x, r.y, r.width, r.height, color)
		end

	drawline (x1, y1, x2, y2: INTEGER; dash: ARRAY[INTEGER]; offset, color: INTEGER) is
		require
			x1 >= 0
			y1 >= 0
			x2 <= width - 1
			y2 <= height - 1
		do
			if dash.is_empty then
				SDL_SURFACE_drawline (to_external, x1, y1, x2, y2, 0, to_external, offset, color)
			else
				SDL_SURFACE_drawline (to_external, x1, y1, x2, y2, dash.count, dash.to_external, offset, color)
			end
		end

	remove_alpha is
			-- Ignore alpha channel
		do
			ext_remove_alpha (to_external)
		end

	enable_alpha is
			-- Enable alpha channel
		do
			ext_enable_alpha (to_external)
		end

	dont_release is
			-- Flag this surface as not releaseable
		do
			dont_release_flag := True
		end

	colorize (r: RECTANGLE; palette: ARRAY[INTEGER]) is
			-- map colors within `r' to those in `palette'
		local
			clip: RECTANGLE
		do
			clip := r*bounds
			SDL_SURFACE_colorize (to_external, clip.x, clip.y, clip.width, clip.height, palette.count, palette.to_external)
		end

feature {SDL_SURFACE} -- Representation

	to_external: POINTER
		-- Pointer to external SDL_Surface

	dont_release_flag: BOOLEAN
		-- when set, don't release this surface on dispose.

feature {NONE} -- SDL external functions

	SDL_IMG_Load (ptr: POINTER): POINTER is
		require
			not ptr.is_null
		external "C use <SDL_image.h>"
		alias "IMG_Load"
		end

	SDL_MapRGB (ptr: POINTER; r, g, b: INTEGER): INTEGER is
		require
			not ptr.is_null
			r.in_range (0, 255)
			g.in_range (0, 255)
			b.in_range (0, 255)
		external "C inline use <SDL.h>"
		alias "(SDL_MapRGB (((SDL_Surface *) $ptr) -> format, $r, $g, $b))"
		end

	SDL_FreeSurface (ptr: POINTER) is
		require
			not ptr.is_null
		external "C use <SDL.h>"
		alias "SDL_FreeSurface"
		end

	ext_remove_alpha (ptr: POINTER) is
		require
			not ptr.is_null
		external "C inline use <SDL.h>"
		alias "SDL_SetAlpha ($ptr, 0, 255)"
		end

	ext_enable_alpha (ptr: POINTER) is
		require
			not ptr.is_null
		external "C inline use <SDL.h>"
		alias "SDL_SetAlpha ($ptr, SDL_SRCALPHA, 255)"
		end

	SDL_SURFACE_create (scr: POINTER; w, h: INTEGER; alpha: BOOLEAN): POINTER is
		require
			not scr.is_null
			is_videomode_set
		external "C"
		alias "SDL_SURFACE_create"
		ensure
			not Result.is_null
		end

	SDL_SURFACE_blit (src: POINTER; dst: POINTER;
		src_x: INTEGER; src_y: INTEGER;
		dst_x: INTEGER; dst_y: INTEGER;
		w: INTEGER; h: INTEGER) is
		require
			not src.is_null
			not dst.is_null
		external "C"
		alias "SDL_SURFACE_blit"
		end

	SDL_SURFACE_fillrect (ptr: POINTER; x, y: INTEGER;
		w, h: INTEGER;
		color: INTEGER) is
		require
			not ptr.is_null
			w >= 0
			h >= 0
		external "C"
		alias "SDL_SURFACE_fillrect"
		end

	SDL_SURFACE_drawline (ptr: POINTER; x1, y1, x2, y2: INTEGER;
		size: INTEGER; dash: POINTER;
		offset: INTEGER; color: INTEGER) is
		require
			not ptr.is_null
		external "C"
		alias "SDL_SURFACE_drawline"
		end

	SDL_SURFACE_get_pixel (ptr: POINTER; x, y: INTEGER): INTEGER is
		require
			not ptr.is_null
		external "C"
		alias "SDL_SURFACE_get_pixel"
		end

	SDL_SURFACE_width (ptr: POINTER): INTEGER is
		require
			not ptr.is_null
		external "C struct SDL_Surface get w use <SDL.h>"
		end

	SDL_SURFACE_height (ptr: POINTER): INTEGER is
		require
			not ptr.is_null
		external "C struct SDL_Surface get h use <SDL.h>"
		end

	SDL_SURFACE_colorize (ptr: POINTER; x, y, w, h, size: INTEGER; palette: POINTER) is
		require
			not ptr.is_null
			not palette.is_null
			size >= 0
			x >= 0 and y >= 0
			x+w <= width and y+h <= height
		external "C use <SDL.h>"
		alias "SDL_SURFACE_colorize"
		end

invariant
	dont_release_flag or else not to_external.is_null
end -- class SDL_SURFACE
