-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
expanded class RECTANGLE
	-- Rectangle with sides parallel to the screen

inherit
	ANY
		redefine fill_tagged_out_memory end

feature -- Operations

	set_with_size (x_1, y_1, w, h: INTEGER) is
			-- Set top-left coordinate and size
		require
			w >= 0 and h >= 0
		do
			x := x_1; y := y_1
			width := w; height := h
		ensure
			x = x_1 and y = y_1
			width = w and height = h
		end

	set_with_coords (x_1, y_1, x_2, y_2: INTEGER) is
			-- Set top-left and bottom-right coordinates
		require
			x_2 >= x_1 - 1 and y_2 >= y_1 - 1
		do
			x := x_1; y := y_1
			width := x_2 - x_1 + 1; height := y_2 - y_1 + 1
		ensure
			x = x_1 and y = y_1
			x2 = x_2 and y2 = y_2
		end

	move_to (x_1, y_1: INTEGER) is
			-- Move upper left corner to (`x_1', `y_1')
		do
			x := x_1
			y := y_1
		ensure
			x = x_1 and y = y_1
		end

	translate (dx, dy: INTEGER) is
			-- Translate by (`dx', `dy')
		do
			x := x + dx
			y := y + dy
		ensure
			x = old x + dx and y = old y + dy
		end

feature -- Access

	x: INTEGER
		-- Left coordinate

	y: INTEGER
		-- Top coordinate

	width, height: INTEGER
		-- Rectangle dimensions

	area: INTEGER is
			-- Rectangle area
		do
			Result := width*height
		end

	x2: INTEGER is
			-- Right coordinate
		do Result := x + width - 1 end

	y2: INTEGER is
			-- Bottom coordinate
		do Result := y + height - 1 end

	has (px, py: INTEGER): BOOLEAN is
			-- Point (`px',`py') is in rectangle
		do
			Result := x <= px and px <= x2 and
				y <= py and py <= y2
		end

	contains (other: RECTANGLE): BOOLEAN is
			-- Completely contains `other' inside?
		do
			Result := (Current+other) = Current
		end

feature -- Operators

	infix "+" (other: RECTANGLE): RECTANGLE is
			-- Smallest which contains `Current' and `other'
		do
			if area=0 then
				Result := other
			elseif other.area = 0 then
				Result := Current
			else
				Result.set_with_coords (x.min (other.x), y.min (other.y),
					x2.max (other.x2), y2.max (other.y2))
			end
		end

	infix "*" (other: RECTANGLE): RECTANGLE is
			-- Intersection of `Current' and `other'
		local
			rx1, ry1: INTEGER
		do
			rx1 := x.max (other.x)
			ry1 := y.max (other.y)
			Result.set_with_coords (rx1, ry1,
				x2.min (other.x2).max (rx1-1),
				y2.min (other.y2).max (ry1-1))
		end

feature -- Redefined features

	fill_tagged_out_memory is
		do
			x.append_in (tagged_out_memory)
			tagged_out_memory.extend (',')
			y.append_in (tagged_out_memory)
			tagged_out_memory.extend (' ')
			width.append_in (tagged_out_memory)
			tagged_out_memory.extend ('x')
			height.append_in (tagged_out_memory)
		end

invariant
	width >= 0 and height >= 0
	x2 = x + width - 1
	y2 = y + height - 1

end -- class RECTANGLE