-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class BUTTON

inherit
	WINDOW
		redefine make, handle_event end

create
	make

feature {NONE} -- Creation

	make (w: WINDOW; where: RECTANGLE) is
		do
			set_state (st_normal)
			Precursor (w, where)
		end

feature -- Operations

	set_click_handler (handler: PROCEDURE [ANY, TUPLE]) is
			-- Set action to do when clicked.
		do
			click_handler := handler
		end

	set_on_enter_handler (handler: PROCEDURE [ANY, TUPLE]) is
			-- Set action to do when clicked.
		do
			on_enter_handler := handler
		end

	set_on_exit_handler (handler: PROCEDURE [ANY, TUPLE]) is
			-- Set action to do when clicked.
		do
			on_exit_handler := handler
		end

feature {NONE} -- Representation

	click_handler: PROCEDURE [ANY, TUPLE]
		-- Callback on click

	on_enter_handler: PROCEDURE [ANY, TUPLE]
		-- Callback on mouse enter

	on_exit_handler: PROCEDURE [ANY, TUPLE]
		-- Callback on mouse exit

feature -- Redefined features

	handle_event (event: EVENT) is
		local
			ev_kind: INTEGER
		do
			Precursor (event)
			if not event.handled then
				ev_kind := translate_event (event)
				if ev_kind /= ev_none then
					set_state (transition (ev_kind))
				end
			end
		end

feature {NONE} -- Internal

	ev_enter, ev_leave, ev_bdown, ev_bup, ev_none: INTEGER is unique
		-- Possible events

	translate_event (event: EVENT): INTEGER is
			-- Given event, return one of the `ev_*' constants
		local
			n: EVENT_MOUSE_NOTIFY
			b: EVENT_MOUSE_BUTTON
		do
			Result := ev_none
			n ?= event
			if n /= Void then
				if n.is_enter then Result := ev_enter else
					Result := ev_leave
				end
			end
			b ?= event
			if b /= Void then
				if b.button = 1 and b.state then
					Result := ev_bdown
					event.set_handled
				elseif b.button = 1 and not b.state then
					Result := ev_bup
					event.set_handled
				end
			end
		end

	state: INTEGER
		-- Button state, one of the `st_*' constants

	st_normal, st_prelight, st_pressed: INTEGER is unique
		-- Possible button state

	set_state (new_state: INTEGER) is
			-- Change state to `new_state'
		require
			valid_new_state: st_normal <= new_state and new_state <= st_pressed
		do
			if
				state = st_pressed and new_state = st_prelight and
				click_handler /= Void
			then
				click_handler.call ([])
			elseif
				state = st_normal and new_state = st_prelight
				and on_enter_handler /= Void
			then
				on_enter_handler.call([])
			elseif
				state = st_prelight and new_state = st_normal
				and on_exit_handler /= Void
			then
				on_exit_handler.call([])
			end
			state := new_state
		ensure
			state = new_state
		end

	transition (ev_kind: INTEGER): INTEGER is
			-- Transition table for FSM
		do
			Result := state
			inspect ev_kind
			when ev_none then
			when ev_enter then
				if state /= st_pressed then Result := st_prelight end
			when ev_leave then Result := st_normal
			when ev_bdown then Result := st_pressed
			when ev_bup then
				if state /= st_normal then Result := st_prelight end
			end
		end

invariant
	valid_state: state.in_range (st_normal, st_pressed)

end -- class BUTTON
