-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
deferred class SCROLLBAR

inherit
	WINDOW
		rename
			make as window_make
		redefine
			redraw
		end

feature -- Access

	value: INTEGER
		-- Scrollbar position

	lower, upper: INTEGER
		-- Limits for visible values

	page_size: INTEGER
		-- Items visible at the same time (0 based)

	step_increment, page_increment: INTEGER
		-- Small and large increment when moving

feature -- Operations


	redraw (area: RECTANGLE) is
		do
			show_image (slidebar, trough.x + slidebar_x, trough.y + slidebar_y, area)
			Precursor (area)
		end

	set_first_button_images (i1, i2, i3: IMAGE) is
			-- Set the images used for the first button
		deferred
		end

	set_second_button_images (i1, i2, i3: IMAGE) is
			-- Set the images used for the first button
		deferred
		end

	set_trough(r: RECTANGLE) is
		do
			trough := r
			resize_slidebar
			request_redraw(r)
		ensure
			trough = r
		end

	set_value (new_value: INTEGER) is
			-- Update `value'
		require
			lower <= upper implies (new_value >= lower and new_value + page_size <= upper)
		do
			value := new_value
				resize_slidebar
			if change_handler /= Void then
				change_handler.call([value])
			end
			request_redraw (trough)
		ensure
			value = new_value
		end

	make_visible (new_value: INTEGER) is
			-- Scroll as little as possible to make `new_value' in range
		require
			new_value.in_range (lower, upper)
		do
			if new_value < value then
				set_value(new_value)
			elseif new_value > value + page_size then
				set_value(new_value - page_size)
			end
		ensure
			new_value.in_range (value, value + page_size)
		end

	set_limits (new_lower, new_upper, new_page_size: INTEGER) is
			-- Update `lower', `upper', `page_size'
		require
			new_lower <= new_upper + 1
			new_page_size >= 0
		do
			lower := new_lower
			upper := new_upper
			page_size := new_page_size
			if lower <= upper then
				if value < lower then
					value := lower
				elseif value + page_size > upper then
					value := upper - page_size
				end
				resize_slidebar
				request_redraw(trough)
			end
		ensure
			lower = new_lower
			upper = new_upper
			page_size = new_page_size
		end

	set_increments (new_step, new_page: INTEGER) is
			-- Update `step_increment' and `page_increment'
		require
			new_step >= 0
			new_page >= 0
		do
			step_increment := new_step
			page_increment := new_page
		ensure
			step_increment = new_step
			page_increment = new_page
		end

	set_change_handler (handler: PROCEDURE [ANY, TUPLE [INTEGER]]) is
			-- Set action when value changes
		do
			change_handler := handler
		end

feature {NONE} -- Representation

	change_handler: PROCEDURE [ANY, TUPLE [INTEGER]]
		-- Callback on value_change

	trough: RECTANGLE
		-- Rectangle within which the slidebar moves

	slidebar: RESIZABLE_IMAGE
		-- Image used for the slidebar

	slidebar_x, slidebar_y: INTEGER
		-- Slidebar Position

	first_button, second_button: BUTTON_IMAGE

feature {NONE} -- Internal

	resize_slidebar is
			-- Recalculates slidebar size and position based on
			-- (possibly new) value, limits and trough size
		deferred
		end

feature {NONE} -- Creation

	make (w: WINDOW; where: RECTANGLE; s:RESIZABLE_IMAGE) is
		require
			w /= Void and s /= Void
		do
			window_make(w, where)
			slidebar := s
		end

invariant
	valid_limits: lower <= upper + 1 and page_size >= 0
	valid_value: lower <= upper implies value.in_range(lower, upper - page_size)
	valid_increments: step_increment >= 0 and page_increment >= 0
end -- class SCROLLBAR
