-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class TEXT_ENTRY

inherit
	TAB_CYCLABLE
		redefine
			make, handle_event, redraw
		end
	TEXT_CONSTANTS

create
	make

feature {NONE} -- Creation

	make (w: WINDOW; where: RECTANGLE) is
		local
			r: RECTANGLE
		do
			create editor.make (create {TEXT_LINE}.make (""))
			Precursor (w, where)
			focused := has_focus
			r := where
			r.move_to (0, 0)
			create label.make (Current, r, "")
			label.set_h_alignment (0)
			label.set_v_alignment (0.5)
			update_label
-- This shouldn't be needed but is. SE bug
			text_hiding := False
		end

feature -- Access

	text: STRING is
			-- Entered text
		do
			Result := editor.text
		end

	border_size: INTEGER

feature -- Operations

	set_default_action (a:PROCEDURE[ANY, TUPLE]) is
			-- Set a procedure to be called when user hits enter
		do
			default_action := a
		end

	set_border_size (new_border_size: INTEGER) is
		do
			border_size := new_border_size
			label.set_border_width (new_border_size)
			update_label
		end

	set_font (new_font: FONT) is
			-- Set text font
		do
			font := new_font
			label.set_font (new_font)
			-- Alignment may have changed, so update is neeeded
			update_label
		end

	set_text (new_text: STRING) is
			-- Set entered text
		do
			editor.set_text (new_text)
			update_label
		end

	enable_text_hiding is
			-- Turn on invisible text mode (for passwords and such).
		do
			text_hiding := True
			update_label
		end

	call_default_action is
			-- Call the default action if one is set
		do
			if default_action /= Void then
				default_action.call([])
			end
		end

feature -- Redefined features

	handle_event (event: EVENT) is
		local
			k: EVENT_KEY
			m: EVENT_MOUSE_BUTTON
			f: EVENT_FOCUS
			dont_update: BOOLEAN
		do
			Precursor (event)
			if not event.handled then
				k ?= event
				m ?= event
				f ?= event
				if k /= Void and then k.state then
					inspect
						k.symbol
					when key_backspace then editor.remove_left
					when key_delete then editor.remove
					when key_right then editor.go_right
					when key_left then editor.go_left
					when key_home then editor.go_home
					when key_end then editor.go_end
					when key_enter then call_default_action
					when key_kp_enter then call_default_action
					when 32..126 then
						if k.character.in_range (32, 126) then
							editor.insert (k.character.to_character)
						end
					else
						dont_update := True
					end
					if not dont_update then update_label end
					k.set_handled
				elseif m /= Void and then m.state then
					grab
					editor.set_position (
						actual_font.max_chars (shown_text, (offset+m.x-border_size).max(0))+1
					)
					update_label
					m.set_handled
				elseif f /= Void then
					focused := f.state
					update_label
				end
			end
		end

	redraw (area: RECTANGLE) is
		local
			i: SDL_SOLID_IMAGE
		do
			Precursor (area)
			-- Show cursor
			if focused then
				create i.make (1, height-2*border_size, 255, 255, 255)
				show_image (i,(curpos-offset+border_size-1).max(0),border_size, area)
			end
		end

feature {NONE} -- Representation

	default_action: PROCEDURE[ANY, TUPLE]

	editor: LINE_EDITOR

	label: LABEL
		-- Actual widget showing the text

	offset, curpos: INTEGER

	font: FONT

	focused: BOOLEAN
		-- to keep focus status

	text_hiding: BOOLEAN
		-- True when text must not be shown

	shown_text: STRING is
		do
			if text_hiding then
				create Result.make_filled ('*', text.count)
			else
				Result := text
			end
		end
	
	update_label is
		-- Change label text when `text' is changed.
		local
			new_text: STRING
			ew: INTEGER -- Effective width
		do
			ew := width - border_size*2
			new_text := shown_text
			if focused then
				curpos := actual_font.width_of (
					new_text.substring (1, editor.effective_position-1))
			end
			label.set_text (new_text)
			if curpos-offset < 0 then
				offset := curpos
			elseif curpos-offset > ew then
				offset := curpos-ew
			end
			-- Align to the right when there is unused space to the right
			if ew<label.text_width and offset+ew>label.text_width then
				offset := label.text_width-ew
			end
			label.set_hoffset (offset)
			request_redraw_all
		end

	actual_font: FONT is
		do
			if font /= Void then
				Result := font
			else
				Result := display.default_font
			end
		end

invariant
	editor /= Void
end
