-- EWS (Eiffel Windowing System) - A small OO windowing system
-- Copyright (C) 2004  Daniel F. Moisset, Anthony Lenton
--
--    This library is free software; you can redistribute it and/or
--    modify it under the terms of the GNU Library General Public
--    License as published by the Free Software Foundation; either
--    version 2 of the License, or (at your option) any later version.
--
--    This library is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--    Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public
--    License along with this library; if not, write to the 
--    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
--    Boston, MA  02111-1307  USA.
--
--  Author's email: dmoisset@grulic.org.ar
--  Snail mail address: Pirovano 196, 5000 Cordoba, Argentina
--
class WINDOW

create
	make, make_root

feature {NONE} -- Creation

	make (w: WINDOW; where: RECTANGLE) is
			-- Make child for `w', placed `where' (relative to `w')
		require
			w /= Void
		do
			display := w.display
			location := where
			parent := w
			create children.make (1, 0)
			parent.add_child (Current)
			show
		end

	make_root (d: DISPLAY) is
			-- Make root window for display `d'
		require
			d.root = Void
		do
			display := d
			location := display.dimensions
			create children.make (1, 0)
			show
		end

feature -- Access

	display: DISPLAY
		-- Display where window is drawn

	pointer: MOUSE_POINTER is
			-- Mouse pointer on this window
		do
			if custom_pointer /= Void then
				Result := custom_pointer
			elseif parent /= Void then
				Result := parent.pointer
			end
		end

	location: RECTANGLE
		-- Location relative to parent

	width: INTEGER is do Result := location.width end
	height: INTEGER is do Result := location.height end

	absolute_location: RECTANGLE is
			-- Location relative to display
		local
			parent_location: RECTANGLE
		do
			if absolute_location_memory_valid then
				Result := absolute_location_memory
			elseif parent = Void then
				Result := location
				absolute_location_memory_valid := True
				absolute_location_memory := Result
			else
--FIXME: Make iterative
				parent_location := parent.absolute_location
				Result := location
				Result.translate (parent_location.x, parent_location.y)

				absolute_location_memory_valid := True
				absolute_location_memory := Result
			end
		end

feature -- Access -- visibility

	visible: BOOLEAN

feature -- Operations -- visibility

	show is
		do
			visible := True
			request_redraw_all
		ensure
			visible
		end

	hide is
		do
			visible := False
			request_redraw_all
		ensure
			not visible
		end

feature -- Access -- focus

	selected_child: WINDOW
		-- child who receives focused events

	has_selection: BOOLEAN is
			-- Parent has Current as selection
		do
			Result := parent = Void or else parent.selected_child = Current
		end

	has_focus: BOOLEAN is
			-- selection chain from the root points to Current
		do
			Result := has_selection and then
				(parent = Void or else parent.has_focus)
		end

feature -- Operations -- focus

	grab is
			-- grab focus
		require
			visible
		do
			if parent /= Void then
				if not parent.has_selection then parent.grab end
				check parent.has_focus end
				parent.select_child (Current)
			end
		ensure
			has_focus
		end

feature {WINDOW} -- Operations -- focus

	select_child (w: WINDOW) is
			-- select `w'
		require
			children.fast_has (w)
		local
			f: BOOLEAN
		do
			f := has_focus
			if selected_child /= Void and then selected_child.visible and f then
				selected_child.handle_event (create {EVENT_FOCUS}.make (False))
			end
			selected_child := w
			if f and selected_child.visible then
				selected_child.handle_event (create {EVENT_FOCUS}.make (True))
			end
		end

feature -- Access -- window tree

	parent: WINDOW
		-- Parent of this window, Void only for root windows

	children: ARRAY [WINDOW]
		-- children windows in z-order (first is higher)

	child_at (x, y: INTEGER): WINDOW is
			-- Proper topmost direct child at (`x',`y'), Void if none
		local
			i: INTEGER -- not using ITERATORs for efficiency reasons
			r: RECTANGLE -- SE bug workaround
		do
			from
				i := children.lower
				if i <= children.upper then r := (children @ i).location end
			until
				i > children.upper or else r.has (x, y)
			loop
				i := i + 1
				if i <= children.upper then r := (children @ i).location end
			end
			if children.valid_index (i) then
				Result := children @ i
			end
		ensure
			Result /= Void implies children.fast_has (Result)
--        Result /= Void implies Result.location.has (x, y) -- SE Bug workaround
		end

	subwindow_at (x, y: INTEGER): WINDOW is
			-- deepest subwindow at (`x',`y'), from Current
		local
			ch: WINDOW
			px, py: INTEGER
		do
			from
				px := x; py := y
				Result := Current
				ch := child_at (px, py)
			until ch = Void loop
				px := px-ch.location.x; py := y-ch.location.y
				Result := ch
				ch := Result.child_at (px, py)
			end
		end

feature -- Operations -- window tree

	remove is
			-- Remove this window
		require
			parent /= Void
		do
			display.add_event (create {EVENT_REMOVE}.make (Current))
		end

	one_up is
				-- Raises the window one Position
		require
				parent /= Void and then parent.children.fast_has(Current)
		local
			pos: INTEGER
		do
			pos := parent.children.fast_index_of(Current)
			if pos > parent.children.lower then
				parent.children.swap(pos, pos - 1)
			end
			request_redraw_all
		ensure
			parent.children.fast_index_of(Current) = old parent.children.fast_index_of(Current) - 1
		end

	one_down is
			-- Lowers the window one Position ('lower' sounded confusing)
		require
			parent /= Void and then parent.children.fast_has(Current)
		local
			pos: INTEGER
		do
			pos := parent.children.fast_index_of(Current)
			if pos < parent.children.upper then
				parent.children.swap(pos, pos + 1)
			end
			request_redraw_all
		ensure
			parent.children.fast_index_of(Current) = old parent.children.fast_index_of(Current) + 1
		end

	bring_to_front is
			-- Brings this window infront of all it's brothers
		require
			parent /= Void and then parent.children.fast_has(Current)
		local
			pos: INTEGER
		do
			pos := parent.children.fast_index_of(Current)
			if pos > parent.children.lower then
				parent.children.remove(pos)
				parent.children.add_first(Current)
			end
		ensure
			parent.children.fast_index_of(Current) = parent.children.lower
		end


	send_to_back is
			-- Brings this window infront of all it's brothers
		require
			parent /= Void and then parent.children.fast_has(Current)
		local
			pos: INTEGER
		do
			pos := parent.children.fast_index_of(Current)
			if pos < parent.children.upper then
				parent.children.remove(pos)
				parent.children.add_last(Current)
			end
		ensure
			parent.children.fast_index_of(Current) = parent.children.upper
		end

	bring_in_front_of(other: WINDOW) is
			-- Brings this window to the first position in front of `other'
		require
			(parent /= Void and Current /= other) and then (parent.children.fast_has(Current) and parent.children.fast_has(other))
		local
		do
			parent.children.remove(parent.children.fast_index_of(Current))
			parent.children.add(Current, parent.children.fast_index_of(other))
		ensure
			parent.children.fast_index_of(Current) = parent.children.fast_index_of(other) - 1
		end

	send_behind(other: WINDOW) is
			-- Sends this window to the first position behind `other'
		require
			(parent /= Void and Current /= other) and then (parent.children.fast_has(Current) and parent.children.fast_has(other))
		local
		do
			parent.children.remove(parent.children.fast_index_of(Current))
			parent.children.add(Current, parent.children.fast_index_of(other) + 1)
		ensure
			parent.children.fast_index_of(Current) = parent.children.fast_index_of(other) + 1
		end

feature {WINDOW} -- Operations -- window tree

	add_child (w: WINDOW) is
			-- Add `w' to child list
		require
			w /= Void
			w.parent = Current
		do
			children.add_first (w)
			if selected_child = Void then
				selected_child := w
			end
		end

feature {EVENT_REMOVE} -- Operations -- window tree

	remove_child (w: WINDOW) is
			-- Remove `w' from child list
		require
			children.fast_has (w)
		local
			r: RECTANGLE
		do
			r.set_with_size (0, 0, width, height)
			request_redraw (r*w.location)
			children.remove (children.fast_index_of (w))
			if mouse_focus = w then mouse_focus := Void end
			if selected_child = w then selected_child := Void end
		end

feature -- Operations -- drawing primitives

	show_image (i: IMAGE; x, y: INTEGER; clip: RECTANGLE) is
			-- Draws `i' at relative (`x',`y'), clipped inside `clip'
			-- Call only from `redraw'
		require
			i /= Void
		local
			imgrect, absloc: RECTANGLE
			ix, iy: INTEGER
		do
			absloc := absolute_location
			imgrect.set_with_size (x, y, i.width, i.height)
			imgrect := (imgrect * clip)
			ix := absloc.x + imgrect.x
			iy := absloc.y + imgrect.y
			imgrect.translate (-x, -y)
			i.show_region (display, imgrect, ix, iy)
		end

feature -- Operations

	set_pointer (new_pointer: MOUSE_POINTER) is
		do
			custom_pointer := new_pointer
			display.update_pointer
		ensure
			new_pointer /= Void implies pointer = new_pointer
		end

	handle_event (event: EVENT) is
			-- Process given event
		require
			event /= Void
			visible
		do
			if not event.handled then
				event.propagate (Current)
			end
		end

	request_redraw (area: RECTANGLE) is
		local
			al: RECTANGLE
		do
			al := absolute_location
			area.translate (al.x, al.y)
			display.request_redraw (area)
		end

	request_redraw_all is
			-- Requests a redraw for whole window
		do
			display.request_redraw (absolute_location)
		end

	redraw (area: RECTANGLE) is
			-- Redraw `area' of self
		require
			visible
		local
			i: INTEGER
			w: WINDOW
			ca: RECTANGLE
			loc: RECTANGLE
		do
			from i := children.upper until i < children.lower loop
				w := children @ i
				if w.visible then
					loc := w.location
					ca := area * loc
					if ca.area > 0 then
						ca.translate (-loc.x, -loc.y)
						w.redraw (ca)
					end
				end
				i := i - 1
			end
		end

feature -- For enter and leave events

	mouse_focus: WINDOW
		-- Window having the mouse focus

	set_mouse_focus (x, y: INTEGER) is
			-- Set the focused window to child at (`x',`y')
		do
			mouse_focus := child_at (x, y)
		end

feature -- Movement

	move (r: RECTANGLE) is
			-- Set `location' to `r'
		do
			request_redraw_all
			location := r
-- FIXME: check for tricky interactions with `mouse_focus'
			invalidate_absolute_location_memory
			request_redraw_all
		end

feature {WINDOW} -- Movement

	invalidate_absolute_location_memory is
		local
			i: ITERATOR [WINDOW]
		do
			absolute_location_memory_valid := False
			i := children.get_new_iterator
			from i.start until i.is_off loop
				i.item.invalidate_absolute_location_memory
				i.next
			end
		end

feature {NONE} -- Internal

	custom_pointer: MOUSE_POINTER

	absolute_location_memory: RECTANGLE
		-- Memory for precomputed `absolute_location'

	absolute_location_memory_valid: BOOLEAN
		-- if `absolute_location_memory' is valid

invariant
	children /= Void
	displayed: display /= Void
	displayed_on_parent: parent /= Void implies display = parent.display
-- Commented out; too expensive to check during redraw
--    mouse_focus_on_child: mouse_focus /= Void implies children.fast_has (mouse_focus)
--    selected_is_child: selected_child /= Void implies children.fast_has (selected_child)

end
